﻿using System.IO;
using ImageProcessingGraph.Editor.Nodes.NodeAttributes;
using UnityEditor;
using UnityEngine;

namespace ImageProcessingGraph.Editor.Nodes.Types.Image.From_Path_Nodes
{
    public class FromPathNode<T> : BaseImageNode where T : UnityEngine.Object
    {
        [NodeAttributes.Input("Path")] public string path;
        [NodeAttributes.Output("Output")] public T output;

        public override void Process()
        {
            if (string.IsNullOrEmpty(path))
            {
                throw new System.ArgumentException("Path is empty or null. Cannot load asset.");
            }

            output = AssetDatabase.LoadAssetAtPath<T>(path);

            if (output == null)
            {
                throw new System.Exception($"Failed to load asset of type {typeof(T)} at path: {path}.");
            }
        }

        public override void CleanUp()
        {
            output = null;
            path = null;
        }
    }
    
    [NodeInfo("Load Texture2D From path", "Load From Path/Texture2D")]
    public class Texture2DFromPath : FromPathNode<Texture2D> { }
    
    [NodeInfo("Load Mesh From path", "Load From Path/Mesh")]
    public class MeshFromPath : FromPathNode<Mesh> { }
    
    [NodeInfo("Load Prefab From path", "Load From Path/Prefab")]
    public class PrefabFromPath : FromPathNode<GameObject> { }
}